@@ -1,3 +1,39 @@
+"""
+Record Arrays
+=============
+Record arrays expose the fields of structured arrays as properties.
+
+Most commonly, ndarrays contain elements of a single type, e.g. floats, integers,
+bools etc.  However, it is possible for elements to be combinations of these,
+such as::
+
+  >>> a = np.array([(1, 2.0), (1, 2.0)], dtype=[('x', int), ('y', float)])
+  >>> a
+  array([(1, 2.0), (1, 2.0)],
+        dtype=[('x', '<i4'), ('y', '<f8')])
+
+Here, each element consists of two fields: x (and int), and y (a float).
+This is known as a structured array.  The different fields are analogous
+to columns in a spread-sheet.  The different fields can be accessed as
+one would a dictionary::
+
+  >>> a['x']
+  array([1, 1])
+
+  >>> a['y']
+  array([ 2.,  2.])
+
+Record arrays allow us to access fields as properties::
+
+  >>> ar = a.view(np.recarray)
+
+  >>> ar.x
+  array([1, 1])
+
+  >>> ar.y
+  array([ 2.,  2.])
+
+"""
 # All of the functions allow formats to be a dtype
 __all__ = ['record', 'recarray', 'format_parser']
 
@@ -37,16 +73,75 @@ def find_duplicate(list):
     """Find duplication in a list, return a list of duplicated elements"""
     dup = []
     for i in range(len(list)):
-        if (list[i] in list[i+1:]):
+        if (list[i] in list[i + 1:]):
             if (list[i] not in dup):
                 dup.append(list[i])
     return dup
 
 class format_parser:
+    """
+    Class to convert formats, names, titles description to a dtype.
+
+    After constructing the format_parser object, the dtype attribute is
+    the converted data-type:
+    ``dtype = format_parser(formats, names, titles).dtype``
+
+    Attributes
+    ----------
+    dtype : dtype
+        The converted data-type.
+
+    Parameters
+    ----------
+    formats : str or list of str
+        The format description, either specified as a string with
+        comma-separated format descriptions in the form ``'f8, i4, a5'``, or
+        a list of format description strings  in the form
+        ``['f8', 'i4', 'a5']``.
+    names : str or list/tuple of str
+        The field names, either specified as a comma-separated string in the
+        form ``'col1, col2, col3'``, or as a list or tuple of strings in the
+        form ``['col1', 'col2', 'col3']``.
+        An empty list can be used, in that case default field names
+        ('f0', 'f1', ...) are used.
+    titles : sequence
+        Sequence of title strings. An empty list can be used to leave titles
+        out.
+    aligned : bool, optional
+        If True, align the fields by padding as the C-compiler would.
+        Default is False.
+    byteorder : str, optional
+        If specified, all the fields will be changed to the
+        provided byte-order.  Otherwise, the default byte-order is
+        used. For all available string specifiers, see `dtype.newbyteorder`.
+
+    See Also
+    --------
+    dtype, typename, sctype2char
+
+    Examples
+    --------
+    >>> np.format_parser(['f8', 'i4', 'a5'], ['col1', 'col2', 'col3'],
+    ...                  ['T1', 'T2', 'T3']).dtype
+    dtype([(('T1', 'col1'), '<f8'), (('T2', 'col2'), '<i4'),
+           (('T3', 'col3'), '|S5')]
+
+    `names` and/or `titles` can be empty lists. If `titles` is an empty list,
+    titles will simply not appear. If `names` is empty, default field names
+    will be used.
+
+    >>> np.format_parser(['f8', 'i4', 'a5'], ['col1', 'col2', 'col3'],
+    ...                  []).dtype
+    dtype([('col1', '<f8'), ('col2', '<i4'), ('col3', '|S5')])
+    >>> np.format_parser(['f8', 'i4', 'a5'], [], []).dtype
+    dtype([('f0', '<f8'), ('f1', '<i4'), ('f2', '|S5')])
+
+    """
     def __init__(self, formats, names, titles, aligned=False, byteorder=None):
         self._parseFormats(formats, aligned)
         self._setfieldnames(names, titles)
         self._createdescr(byteorder)
+        self.dtype = self._descr
 
     def _parseFormats(self, formats, aligned=0):
         """ Parse the field formats """
@@ -101,7 +196,7 @@ class format_parser:
             titles = []
 
         if (self._nfields > len(titles)):
-            self._titles += [None]*(self._nfields-len(titles))
+            self._titles += [None] * (self._nfields - len(titles))
 
     def _createdescr(self, byteorder):
         descr = sb.dtype({'names':self._names,
@@ -115,6 +210,8 @@ class format_parser:
         self._descr = descr
 
 class record(nt.void):
+    """A data-type scalar that allows field access as attribute lookup.
+    """
     def __repr__(self):
         return self.__str__()
 
@@ -152,26 +249,26 @@ class record(nt.void):
     def __setattr__(self, attr, val):
         if attr in ['setfield', 'getfield', 'dtype']:
             raise AttributeError, "Cannot set '%s' attribute" % attr
-        try:
-            return nt.void.__setattr__(self, attr, val)
-        except AttributeError:
-            pass
         fielddict = nt.void.__getattribute__(self, 'dtype').fields
         res = fielddict.get(attr, None)
         if res:
             return self.setfield(val, *res[:2])
         else:
-            raise AttributeError, "'record' object has no "\
-                  "attribute '%s'" % attr
+            if getattr(self, attr, None):
+                return nt.void.__setattr__(self, attr, val)
+            else:
+                raise AttributeError, "'record' object has no "\
+                      "attribute '%s'" % attr
 
     def pprint(self):
+        """Pretty-print all fields."""
         # pretty-print all fields
         names = self.dtype.names
         maxlen = max([len(name) for name in names])
         rows = []
-        fmt = '%% %ds: %%s' %maxlen
+        fmt = '%% %ds: %%s' % maxlen
         for name in names:
-            rows.append(fmt%(name, getattr(self, name)))
+            rows.append(fmt % (name, getattr(self, name)))
         return "\n".join(rows)
 
 # The recarray is almost identical to a standard array (which supports
@@ -183,6 +280,109 @@ class record(nt.void):
 #  the fields (and any subfields)
 
 class recarray(ndarray):
+    """
+    Construct an ndarray that allows field access using attributes.
+
+    Arrays may have a data-types containing fields, analagous
+    to columns in a spread sheet.  An example is ``[(x, int), (y, float)]``,
+    where each entry in the array is a pair of ``(int, float)``.  Normally,
+    these attributes are accessed using dictionary lookups such as ``arr['x']``
+    and ``arr['y']``.  Record arrays allow the fields to be accessed as members
+    of the array, using ``arr.x`` and ``arr.y``.
+
+    Parameters
+    ----------
+    shape : tuple
+        Shape of output array.
+    dtype : data-type, optional
+        The desired data-type.  By default, the data-type is determined
+        from `formats`, `names`, `titles`, `aligned` and `byteorder`.
+    formats : list of data-types, optional
+        A list containing the data-types for the different columns, e.g.
+        ``['i4', 'f8', 'i4']``.  `formats` does *not* support the new
+        convention of using types directly, i.e. ``(int, float, int)``.
+        Note that `formats` must be a list, not a tuple.
+        Given that `formats` is somewhat limited, we recommend specifying
+        `dtype` instead.
+    names : tuple of str, optional
+        The name of each column, e.g. ``('x', 'y', 'z')``.
+    buf : buffer, optional
+        By default, a new array is created of the given shape and data-type.
+        If `buf` is specified and is an object exposing the buffer interface,
+        the array will use the memory from the existing buffer.  In this case,
+        the `offset` and `strides` keywords are available.
+
+    Other Parameters
+    ----------------
+    titles : tuple of str, optional
+        Aliases for column names.  For example, if `names` were
+        ``('x', 'y', 'z')`` and `titles` is
+        ``('x_coordinate', 'y_coordinate', 'z_coordinate')``, then
+        ``arr['x']`` is equivalent to both ``arr.x`` and ``arr.x_coordinate``.
+    byteorder : {'<', '>', '='}, optional
+        Byte-order for all fields.
+    aligned : bool, optional
+        Align the fields in memory as the C-compiler would.
+    strides : tuple of ints, optional
+        Buffer (`buf`) is interpreted according to these strides (strides
+        define how many bytes each array element, row, column, etc.
+        occupy in memory).
+    offset : int, optional
+        Start reading buffer (`buf`) from this offset onwards.
+
+    Returns
+    -------
+    rec : recarray
+        Empty array of the given shape and type.
+
+    See Also
+    --------
+    rec.fromrecords : Construct a record array from data.
+    record : fundamental data-type for `recarray`.
+    format_parser : determine a data-type from formats, names, titles.
+
+    Notes
+    -----
+    This constructor can be compared to ``empty``: it creates a new record
+    array but does not fill it with data.  To create a reccord array from data,
+    use one of the following methods:
+
+    1. Create a standard ndarray and convert it to a record array,
+       using ``arr.view(np.recarray)``
+    2. Use the `buf` keyword.
+    3. Use `np.rec.fromrecords`.
+
+    Examples
+    --------
+    Create an array with two fields, ``x`` and ``y``:
+
+    >>> x = np.array([(1.0, 2), (3.0, 4)], dtype=[('x', float), ('y', int)])
+    >>> x
+    array([(1.0, 2), (3.0, 4)],
+          dtype=[('x', '<f8'), ('y', '<i4')])
+
+    >>> x['x']
+    array([ 1.,  3.])
+
+    View the array as a record array:
+
+    >>> x = x.view(np.recarray)
+
+    >>> x.x
+    array([ 1.,  3.])
+
+    >>> x.y
+    array([2, 4])
+
+    Create a new, empty record array:
+
+    >>> np.recarray((2,),
+    ... dtype=[('x', int), ('y', float), ('z', int)]) #doctest: +SKIP
+    rec.array([(-1073741821, 1.2249118382103472e-301, 24547520),
+           (3471280, 1.2134086255804012e-316, 0)],
+          dtype=[('x', '<i4'), ('y', '<f8'), ('z', '<i4')])
+
+    """
     def __new__(subtype, shape, dtype=None, buf=None, offset=0, strides=None,
                 formats=None, names=None, titles=None,
                 byteorder=None, aligned=False):
@@ -205,7 +405,7 @@ class recarray(ndarray):
             return object.__getattribute__(self, attr)
         except AttributeError: # attr must be a fieldname
             pass
-        fielddict = ndarray.__getattribute__(self,'dtype').fields
+        fielddict = ndarray.__getattribute__(self, 'dtype').fields
         try:
             res = fielddict[attr][:2]
         except (TypeError, KeyError):
@@ -229,12 +429,12 @@ class recarray(ndarray):
         try:
             ret = object.__setattr__(self, attr, val)
         except:
-            fielddict = ndarray.__getattribute__(self,'dtype').fields or {}
+            fielddict = ndarray.__getattribute__(self, 'dtype').fields or {}
             if attr not in fielddict:
                 exctype, value = sys.exc_info()[:2]
                 raise exctype, value
         else:
-            fielddict = ndarray.__getattribute__(self,'dtype').fields or {}
+            fielddict = ndarray.__getattribute__(self, 'dtype').fields or {}
             if attr not in fielddict:
                 return ret
             if newattr:         # We just added this one
@@ -245,7 +445,7 @@ class recarray(ndarray):
                     return ret
         try:
             res = fielddict[attr][:2]
-        except (TypeError,KeyError):
+        except (TypeError, KeyError):
             raise AttributeError, "record array has no attribute %s" % attr
         return self.setfield(val, *res)
 
@@ -255,12 +455,16 @@ class recarray(ndarray):
             return obj.view(ndarray)
         return obj
 
+    def __repr__(self) :
+        ret = ndarray.__repr__(self)
+        return ret.replace("recarray", "rec.array", 1)
+
     def field(self, attr, val=None):
         if isinstance(attr, int):
-            names = ndarray.__getattribute__(self,'dtype').names
+            names = ndarray.__getattribute__(self, 'dtype').names
             attr = names[attr]
 
-        fielddict = ndarray.__getattribute__(self,'dtype').fields
+        fielddict = ndarray.__getattribute__(self, 'dtype').fields
 
         res = fielddict[attr][:2]
 
@@ -274,25 +478,31 @@ class recarray(ndarray):
         else:
             return self.setfield(val, *res)
 
-    def view(self, obj):
-        try:
-            if issubclass(obj, ndarray):
-                return ndarray.view(self, obj)
-        except TypeError:
-            pass
-        dtype = sb.dtype(obj)
-        if dtype.fields is None:
-            return self.__array__().view(dtype)
-        return ndarray.view(self, obj)
+    def view(self, dtype=None, type=None):
+        if dtype is None:
+            return ndarray.view(self, type)
+        elif type is None:
+            try:
+                if issubclass(dtype, ndarray):
+                    return ndarray.view(self, dtype)
+            except TypeError:
+                pass
+            dtype = sb.dtype(dtype)
+            if dtype.fields is None:
+                return self.__array__().view(dtype)
+            return ndarray.view(self, dtype)
+        else:
+            return ndarray.view(self, dtype, type)
+
 
 def fromarrays(arrayList, dtype=None, shape=None, formats=None,
                names=None, titles=None, aligned=False, byteorder=None):
     """ create a record array from a (flat) list of arrays
 
-    >>> x1=N.array([1,2,3,4])
-    >>> x2=N.array(['a','dd','xyz','12'])
-    >>> x3=N.array([1.1,2,3,4])
-    >>> r = fromarrays([x1,x2,x3],names='a,b,c')
+    >>> x1=np.array([1,2,3,4])
+    >>> x2=np.array(['a','dd','xyz','12'])
+    >>> x3=np.array([1.1,2,3,4])
+    >>> r = np.core.records.fromarrays([x1,x2,x3],names='a,b,c')
     >>> print r[1]
     (2, 'dd', 2.0)
     >>> x1[1]=34
@@ -341,7 +551,7 @@ def fromarrays(arrayList, dtype=None, shape=None, formats=None,
 
     for k, obj in enumerate(arrayList):
         nn = len(descr[k].shape)
-        testshape = obj.shape[:len(obj.shape)-nn]
+        testshape = obj.shape[:len(obj.shape) - nn]
         if testshape != shape:
             raise ValueError, "array-shape mismatch in array %d" % k
 
@@ -370,13 +580,14 @@ def fromrecords(recList, dtype=None, shape=None, formats=None, names=None,
         If formats is None, then this will auto-detect formats. Use list of
         tuples rather than list of lists for faster processing.
 
-    >>> r=fromrecords([(456,'dbe',1.2),(2,'de',1.3)],names='col1,col2,col3')
+    >>> r=np.core.records.fromrecords([(456,'dbe',1.2),(2,'de',1.3)],
+    ... names='col1,col2,col3')
     >>> print r[0]
     (456, 'dbe', 1.2)
     >>> r.col1
     array([456,   2])
     >>> r.col2
-    chararray(['dbe', 'de'], 
+    chararray(['dbe', 'de'],
           dtype='|S3')
     >>> import cPickle
     >>> print cPickle.loads(cPickle.dumps(r))
@@ -386,17 +597,17 @@ def fromrecords(recList, dtype=None, shape=None, formats=None, names=None,
     nfields = len(recList[0])
     if formats is None and dtype is None:  # slower
         obj = sb.array(recList, dtype=object)
-        arrlist = [sb.array(obj[...,i].tolist()) for i in xrange(nfields)]
+        arrlist = [sb.array(obj[..., i].tolist()) for i in xrange(nfields)]
         return fromarrays(arrlist, formats=formats, shape=shape, names=names,
                           titles=titles, aligned=aligned, byteorder=byteorder)
 
     if dtype is not None:
-        descr = sb.dtype(dtype)
+        descr = sb.dtype((record, dtype))
     else:
         descr = format_parser(formats, names, titles, aligned, byteorder)._descr
 
     try:
-        retval = sb.array(recList, dtype = descr)
+        retval = sb.array(recList, dtype=descr)
     except TypeError:  # list of lists instead of list of tuples
         if (shape is None or shape == 0):
             shape = len(recList)
@@ -414,7 +625,6 @@ def fromrecords(recList, dtype=None, shape=None, formats=None, names=None,
 
     res = retval.view(recarray)
 
-    res.dtype = sb.dtype((record, res.dtype))
     return res
 
 
@@ -434,7 +644,7 @@ def fromstring(datastring, dtype=None, shape=None, offset=0, formats=None,
 
     itemsize = descr.itemsize
     if (shape is None or shape == 0 or shape == -1):
-        shape = (len(datastring)-offset) / itemsize
+        shape = (len(datastring) - offset) / itemsize
 
     _array = recarray(shape, descr, buf=datastring, offset=offset)
     return _array
@@ -456,7 +666,7 @@ def fromfile(fd, dtype=None, shape=None, offset=0, formats=None,
     to be a file object.
 
     >>> from tempfile import TemporaryFile
-    >>> a = N.empty(10,dtype='f8,i4,a5')
+    >>> a = np.empty(10,dtype='f8,i4,a5')
     >>> a[5] = (0.5,10,'abcde')
     >>>
     >>> fd=TemporaryFile()
@@ -464,7 +674,8 @@ def fromfile(fd, dtype=None, shape=None, offset=0, formats=None,
     >>> a.tofile(fd)
     >>>
     >>> fd.seek(0)
-    >>> r=fromfile(fd, formats='f8,i4,a5', shape=10, byteorder='<')
+    >>> r=np.core.records.fromfile(fd, formats='f8,i4,a5', shape=10,
+    ... byteorder='<')
     >>> print r[5]
     (0.5, 10, 'abcde')
     >>> r.shape
@@ -492,14 +703,14 @@ def fromfile(fd, dtype=None, shape=None, offset=0, formats=None,
     itemsize = descr.itemsize
 
     shapeprod = sb.array(shape).prod()
-    shapesize = shapeprod*itemsize
+    shapesize = shapeprod * itemsize
     if shapesize < 0:
         shape = list(shape)
         shape[ shape.index(-1) ] = size / -shapesize
         shape = tuple(shape)
         shapeprod = sb.array(shape).prod()
 
-    nbytes = shapeprod*itemsize
+    nbytes = shapeprod * itemsize
 
     if nbytes > size:
         raise ValueError(
@@ -583,7 +794,7 @@ def array(obj, dtype=None, shape=None, offset=0, strides=None, formats=None,
         obj = sb.array(obj)
         if dtype is not None and (obj.dtype != dtype):
             obj = obj.view(dtype)
-        res  = obj.view(recarray)
+        res = obj.view(recarray)
         if issubclass(res.dtype.type, nt.void):
             res.dtype = sb.dtype((record, res.dtype))
         return res
