@@ -1,28 +1,64 @@
-""" Build swig, f2py, weave, sources.
+""" Build swig, f2py, pyrex sources.
 """
 
 import os
 import re
 import sys
+import shlex
+import copy
 
 from distutils.command import build_ext
 from distutils.dep_util import newer_group, newer
 from distutils.util import get_platform
+from distutils.errors import DistutilsError, DistutilsSetupError
 
+def have_pyrex():
+    try:
+        import Pyrex.Compiler.Main
+        return True
+    except ImportError:
+        return False
+
+# this import can't be done here, as it uses numpy stuff only available
+# after it's installed
+#import numpy.f2py
 from numpy.distutils import log
 from numpy.distutils.misc_util import fortran_ext_match, \
-     appendpath, is_string, is_sequence
+     appendpath, is_string, is_sequence, get_cmd
 from numpy.distutils.from_template import process_file as process_f_file
 from numpy.distutils.conv_template import process_file as process_c_file
 
+def subst_vars(target, source, d):
+    """Substitute any occurence of @foo@ by d['foo'] from source file into
+    target."""
+    var = re.compile('@([a-zA-Z_]+)@')
+    fs = open(source, 'r')
+    try:
+        ft = open(target, 'w')
+        try:
+            for l in fs.readlines():
+                m = var.search(l)
+                if m:
+                    ft.write(l.replace('@%s@' % m.group(1), d[m.group(1)]))
+                else:
+                    ft.write(l)
+        finally:
+            ft.close()
+    finally:
+        fs.close()
+
 class build_src(build_ext.build_ext):
 
     description = "build sources from SWIG, F2PY files or a function"
 
     user_options = [
         ('build-src=', 'd', "directory to \"build\" sources to"),
-        ('f2pyflags=', None, "additonal flags to f2py"),
-        ('swigflags=', None, "additional flags to swig"),
+        ('f2py-opts=', None, "list of f2py command line options"),
+        ('swig=', None, "path to the SWIG executable"),
+        ('swig-opts=', None, "list of SWIG command line options"),
+        ('swig-cpp', None, "make SWIG create C++ files (default is autodetected from sources)"),
+        ('f2pyflags=', None, "additional flags to f2py (use --f2py-opts= instead)"), # obsolete
+        ('swigflags=', None, "additional flags to swig (use --swig-opts= instead)"), # obsolete
         ('force', 'f', "forcibly build everything (ignore file timestamps)"),
         ('inplace', 'i',
          "ignore build-lib and put compiled extensions into the source " +
@@ -44,9 +80,12 @@ class build_src(build_ext.build_ext):
         self.force = None
         self.inplace = None
         self.package_dir = None
-        self.f2pyflags = None
-        self.swigflags = None
-        return
+        self.f2pyflags = None # obsolete
+        self.f2py_opts = None
+        self.swigflags = None # obsolete
+        self.swig_opts = None
+        self.swig_cpp = None
+        self.swig = None
 
     def finalize_options(self):
         self.set_undefined_options('build',
@@ -63,36 +102,60 @@ class build_src(build_ext.build_ext):
         if self.build_src is None:
             plat_specifier = ".%s-%s" % (get_platform(), sys.version[0:3])
             self.build_src = os.path.join(self.build_base, 'src'+plat_specifier)
-        if self.inplace is None:
-            build_ext = self.get_finalized_command('build_ext')
-            self.inplace = build_ext.inplace
 
         # py_modules_dict is used in build_py.find_package_modules
         self.py_modules_dict = {}
 
-        if self.f2pyflags is None:
-            self.f2pyflags = []
+        if self.f2pyflags:
+            if self.f2py_opts:
+                log.warn('ignoring --f2pyflags as --f2py-opts already used')
+            else:
+                self.f2py_opts = self.f2pyflags
+            self.f2pyflags = None
+        if self.f2py_opts is None:
+            self.f2py_opts = []
         else:
-            self.f2pyflags = self.f2pyflags.split() # XXX spaces??
+            self.f2py_opts = shlex.split(self.f2py_opts)
 
-        if self.swigflags is None:
-            self.swigflags = []
+        if self.swigflags:
+            if self.swig_opts:
+                log.warn('ignoring --swigflags as --swig-opts already used')
+            else:
+                self.swig_opts = self.swigflags
+            self.swigflags = None
+
+        if self.swig_opts is None:
+            self.swig_opts = []
         else:
-            self.swigflags = self.swigflags.split() # XXX spaces??
-        return
+            self.swig_opts = shlex.split(self.swig_opts)
+
+        # use options from build_ext command
+        build_ext = self.get_finalized_command('build_ext')
+        if self.inplace is None:
+            self.inplace = build_ext.inplace
+        if self.swig_cpp is None:
+            self.swig_cpp = build_ext.swig_cpp
+        for c in ['swig','swig_opt']:
+            o = '--'+c.replace('_','-')
+            v = getattr(build_ext,c,None)
+            if v:
+                if getattr(self,c):
+                    log.warn('both build_src and build_ext define %s option' % (o))
+                else:
+                    log.info('using "%s=%s" option from build_ext command' % (o,v))
+                    setattr(self, c, v)
 
     def run(self):
+        log.info("build_src")
         if not (self.extensions or self.libraries):
             return
         self.build_sources()
 
-        return
-
     def build_sources(self):
 
         if self.inplace:
-            self.get_package_dir = self.get_finalized_command('build_py')\
-                                   .get_package_dir
+            self.get_package_dir = \
+                     self.get_finalized_command('build_py').get_package_dir
 
         self.build_py_modules_sources()
 
@@ -106,8 +169,7 @@ class build_src(build_ext.build_ext):
                 self.build_extension_sources(ext)
 
         self.build_data_files_sources()
-
-        return
+        self.build_npy_pkg_config()
 
     def build_data_files_sources(self):
         if not self.data_files:
@@ -141,9 +203,63 @@ class build_src(build_ext.build_ext):
                 filenames = get_data_files((d,files))
                 new_data_files.append((d, filenames))
             else:
-                raise
+                raise TypeError(repr(data))
         self.data_files[:] = new_data_files
-        return
+
+
+    def _build_npy_pkg_config(self, info, gd):
+        import shutil
+        template, install_dir, subst_dict = info
+        template_dir = os.path.dirname(template)
+        for k, v in gd.items():
+            subst_dict[k] = v
+
+        if self.inplace == 1:
+            generated_dir = os.path.join(template_dir, install_dir)
+        else:
+            generated_dir = os.path.join(self.build_src, template_dir,
+                    install_dir)
+        generated = os.path.basename(os.path.splitext(template)[0])
+        generated_path = os.path.join(generated_dir, generated)
+        if not os.path.exists(generated_dir):
+            os.makedirs(generated_dir)
+
+        subst_vars(generated_path, template, subst_dict)
+
+        # Where to install relatively to install prefix
+        full_install_dir = os.path.join(template_dir, install_dir)
+        return full_install_dir, generated_path
+
+    def build_npy_pkg_config(self):
+        log.info('build_src: building npy-pkg config files')
+
+        # XXX: another ugly workaround to circumvent distutils brain damage. We
+        # need the install prefix here, but finalizing the options of the
+        # install command when only building sources cause error. Instead, we
+        # copy the install command instance, and finalize the copy so that it
+        # does not disrupt how distutils want to do things when with the
+        # original install command instance.
+        install_cmd = copy.copy(get_cmd('install'))
+        if not install_cmd.finalized == 1:
+            install_cmd.finalize_options()
+        build_npkg = False
+        gd = {}
+        if self.inplace == 1:
+            top_prefix = '.'
+            build_npkg = True
+        elif hasattr(install_cmd, 'install_libbase'):
+            top_prefix = install_cmd.install_libbase
+            build_npkg = True
+
+        if build_npkg:
+            for pkg, infos in self.distribution.installed_pkg_config.items():
+                pkg_path = self.distribution.package_dir[pkg]
+                prefix = os.path.join(os.path.abspath(top_prefix), pkg_path)
+                d = {'prefix': prefix}
+                for info in infos:
+                    install_dir, generated = self._build_npy_pkg_config(info, d)
+                    self.distribution.data_files.append((install_dir,
+                        [generated]))
 
     def build_py_modules_sources(self):
         if not self.py_modules:
@@ -164,13 +280,12 @@ class build_src(build_ext.build_ext):
                 if source is None:
                     continue
                 modules = [(package, module_base, source)]
-                if not self.py_modules_dict.has_key(package):
+                if package not in self.py_modules_dict:
                     self.py_modules_dict[package] = []
                 self.py_modules_dict[package] += modules
             else:
                 new_py_modules.append(source)
         self.py_modules[:] = new_py_modules
-        return
 
     def build_library_sources(self, lib_name, build_info):
         sources = list(build_info.get('sources',[]))
@@ -187,7 +302,8 @@ class build_src(build_ext.build_ext):
         sources, h_files = self.filter_h_files(sources)
 
         if h_files:
-            print self.package,'- nothing done with h_files=',h_files
+            log.info('%s - nothing done with h_files = %s',
+                     self.package, h_files)
 
         #for f in h_files:
         #    self.distribution.headers.append((lib_name,f))
@@ -221,7 +337,7 @@ class build_src(build_ext.build_ext):
 
         sources, py_files = self.filter_py_files(sources)
 
-        if not self.py_modules_dict.has_key(package):
+        if package not in self.py_modules_dict:
             self.py_modules_dict[package] = []
         modules = []
         for f in py_files:
@@ -232,14 +348,13 @@ class build_src(build_ext.build_ext):
         sources, h_files = self.filter_h_files(sources)
 
         if h_files:
-            print package,'- nothing done with h_files=',h_files
+            log.info('%s - nothing done with h_files = %s',
+                     package, h_files)
         #for f in h_files:
         #    self.distribution.headers.append((package,f))
 
         ext.sources = sources
 
-        return
-
     def generate_sources(self, sources, extension):
         new_sources = []
         func_sources = []
@@ -255,7 +370,7 @@ class build_src(build_ext.build_ext):
         else:
             if is_sequence(extension):
                 name = extension[0]
-            #    if not extension[1].has_key('include_dirs'):
+            #    if 'include_dirs' not in extension[1]:
             #        extension[1]['include_dirs'] = []
             #    incl_dirs = extension[1]['include_dirs']
             else:
@@ -334,44 +449,49 @@ class build_src(build_ext.build_ext):
         return new_sources
 
     def pyrex_sources(self, sources, extension):
-        have_pyrex = False
-        try:
-            import Pyrex
-            have_pyrex = True
-        except ImportError:
-            pass
         new_sources = []
         ext_name = extension.name.split('.')[-1]
         for source in sources:
             (base, ext) = os.path.splitext(source)
             if ext == '.pyx':
-                if self.inplace or not have_pyrex:
-                    target_dir = os.path.dirname(base)
-                else:
-                    target_dir = appendpath(self.build_src, os.path.dirname(base))
-                target_file = os.path.join(target_dir, ext_name + '.c')
-                depends = [source] + extension.depends
-                if (self.force or newer_group(depends, target_file, 'newer')):
-                    if have_pyrex:
-                        log.info("pyrexc:> %s" % (target_file))
-                        self.mkpath(target_dir)
-                        from Pyrex.Compiler import Main
-                        options = Main.CompilationOptions(
-                            defaults=Main.default_options,
-                            output_file=target_file)
-                        pyrex_result = Main.compile(source, options=options)
-                        if pyrex_result.num_errors != 0:
-                            raise RuntimeError("%d errors in Pyrex compile" %
-                                               pyrex_result.num_errors)
-                    else:
-                        log.warn("Pyrex needed to compile %s but not available."\
-                                 " Using old target %s"\
-                                 % (source, target_file))
+                target_file = self.generate_a_pyrex_source(base, ext_name,
+                                                           source,
+                                                           extension)
                 new_sources.append(target_file)
             else:
                 new_sources.append(source)
         return new_sources
 
+    def generate_a_pyrex_source(self, base, ext_name, source, extension):
+        if self.inplace or not have_pyrex():
+            target_dir = os.path.dirname(base)
+        else:
+            target_dir = appendpath(self.build_src, os.path.dirname(base))
+        target_file = os.path.join(target_dir, ext_name + '.c')
+        depends = [source] + extension.depends
+        if self.force or newer_group(depends, target_file, 'newer'):
+            if have_pyrex():
+                import Pyrex.Compiler.Main
+                log.info("pyrexc:> %s" % (target_file))
+                self.mkpath(target_dir)
+                options = Pyrex.Compiler.Main.CompilationOptions(
+                    defaults=Pyrex.Compiler.Main.default_options,
+                    include_path=extension.include_dirs,
+                    output_file=target_file)
+                pyrex_result = Pyrex.Compiler.Main.compile(source,
+                                                           options=options)
+                if pyrex_result.num_errors != 0:
+                    raise DistutilsError,"%d errors while compiling %r with Pyrex" \
+                          % (pyrex_result.num_errors, source)
+            elif os.path.isfile(target_file):
+                log.warn("Pyrex required for compiling %r but not available,"\
+                         " using old target %r"\
+                         % (source, target_file))
+            else:
+                raise DistutilsError("Pyrex required for compiling %r"\
+                                     " but notavailable" % (source,))
+        return target_file
+
     def f2py_sources(self, sources, extension):
         new_sources = []
         f2py_sources = []
@@ -391,9 +511,9 @@ class build_src(build_ext.build_ext):
                 if os.path.isfile(source):
                     name = get_f2py_modulename(source)
                     if name != ext_name:
-                        raise ValueError('mismatch of extension names: %s '
-                                         'provides %r but expected %r' % (
-                                          source, name, ext_name))
+                        raise DistutilsSetupError('mismatch of extension names: %s '
+                                                  'provides %r but expected %r' % (
+                            source, name, ext_name))
                     target_file = os.path.join(target_dir,name+'module.c')
                 else:
                     log.debug('  source %s does not exist: skipping f2py\'ing.' \
@@ -402,16 +522,16 @@ class build_src(build_ext.build_ext):
                     skip_f2py = 1
                     target_file = os.path.join(target_dir,name+'module.c')
                     if not os.path.isfile(target_file):
-                        log.debug('  target %s does not exist:\n   '\
-                                  'Assuming %smodule.c was generated with '\
-                                  '"build_src --inplace" command.' \
-                                  % (target_file, name))
+                        log.warn('  target %s does not exist:\n   '\
+                                 'Assuming %smodule.c was generated with '\
+                                 '"build_src --inplace" command.' \
+                                 % (target_file, name))
                         target_dir = os.path.dirname(base)
                         target_file = os.path.join(target_dir,name+'module.c')
                         if not os.path.isfile(target_file):
-                            raise ValueError("%r missing" % (target_file,))
-                        log.debug('   Yes! Using %s as up-to-date target.' \
-                                  % (target_file))
+                            raise DistutilsSetupError("%r missing" % (target_file,))
+                        log.info('   Yes! Using %r as up-to-date target.' \
+                                 % (target_file))
                 target_dirs.append(target_dir)
                 f2py_sources.append(source)
                 f2py_targets[source] = target_file
@@ -424,9 +544,10 @@ class build_src(build_ext.build_ext):
         if not (f2py_sources or f_sources):
             return new_sources
 
-        map(self.mkpath, target_dirs)
+        for d in target_dirs:
+            self.mkpath(d)
 
-        f2py_options = extension.f2py_options + self.f2pyflags
+        f2py_options = extension.f2py_options + self.f2py_opts
 
         if self.distribution.libraries:
             for name,build_info in self.distribution.libraries:
@@ -437,7 +558,7 @@ class build_src(build_ext.build_ext):
 
         if f2py_sources:
             if len(f2py_sources) != 1:
-                raise ValueError(
+                raise DistutilsSetupError(
                     'only one .pyf file is allowed per extension module but got'\
                     ' more: %r' % (f2py_sources,))
             source = f2py_sources[0]
@@ -447,8 +568,9 @@ class build_src(build_ext.build_ext):
             if (self.force or newer_group(depends, target_file,'newer')) \
                    and not skip_f2py:
                 log.info("f2py: %s" % (source))
-                import numpy.f2py as f2py2e
-                f2py2e.run_main(f2py_options + ['--build-dir',target_dir,source])
+                import numpy.f2py
+                numpy.f2py.run_main(f2py_options
+                                    + ['--build-dir',target_dir,source])
             else:
                 log.debug("  skipping '%s' f2py interface (up-to-date)" % (source))
         else:
@@ -463,10 +585,10 @@ class build_src(build_ext.build_ext):
             depends = f_sources + extension.depends
             if (self.force or newer_group(depends, target_file, 'newer')) \
                    and not skip_f2py:
-                import numpy.f2py as f2py2e
                 log.info("f2py:> %s" % (target_file))
                 self.mkpath(target_dir)
-                f2py2e.run_main(f2py_options + ['--lower',
+                import numpy.f2py
+                numpy.f2py.run_main(f2py_options + ['--lower',
                                                 '--build-dir',target_dir]+\
                                 ['-m',ext_name]+f_sources)
             else:
@@ -474,7 +596,7 @@ class build_src(build_ext.build_ext):
                           % (target_file))
 
         if not os.path.isfile(target_file):
-            raise ValueError("%r missing" % (target_file,))
+            raise DistutilsError("f2py target file %r not generated" % (target_file,))
 
         target_c = os.path.join(self.build_src,'fortranobject.c')
         target_h = os.path.join(self.build_src,'fortranobject.h')
@@ -486,8 +608,8 @@ class build_src(build_ext.build_ext):
             extension.include_dirs.append(self.build_src)
 
         if not skip_f2py:
-            import numpy.f2py as f2py2e
-            d = os.path.dirname(f2py2e.__file__)
+            import numpy.f2py
+            d = os.path.dirname(numpy.f2py.__file__)
             source_c = os.path.join(d,'src','fortranobject.c')
             source_h = os.path.join(d,'src','fortranobject.h')
             if newer(source_c,target_c) or newer(source_h,target_h):
@@ -496,9 +618,9 @@ class build_src(build_ext.build_ext):
                 self.copy_file(source_h,target_h)
         else:
             if not os.path.isfile(target_c):
-                raise ValueError("%r missing" % (target_c,))
+                raise DistutilsSetupError("f2py target_c file %r not found" % (target_c,))
             if not os.path.isfile(target_h):
-                raise ValueError("%r missing" % (target_h,))
+                raise DistutilsSetupError("f2py target_h file %r not found" % (target_h,))
 
         for name_ext in ['-f2pywrappers.f','-f2pywrappers2.f90']:
             filename = os.path.join(target_dir,ext_name + name_ext)
@@ -518,8 +640,12 @@ class build_src(build_ext.build_ext):
         target_dirs = []
         py_files = []     # swig generated .py files
         target_ext = '.c'
-        typ = None
-        is_cpp = 0
+        if self.swig_cpp:
+            typ = 'c++'
+            is_cpp = True
+        else:
+            typ = None
+            is_cpp = False
         skip_swig = 0
         ext_name = extension.name.split('.')[-1]
 
@@ -535,35 +661,43 @@ class build_src(build_ext.build_ext):
                 if os.path.isfile(source):
                     name = get_swig_modulename(source)
                     if name != ext_name[1:]:
-                        raise ValueError(
+                        raise DistutilsSetupError(
                             'mismatch of extension names: %s provides %r'
                             ' but expected %r' % (source, name, ext_name[1:]))
                     if typ is None:
                         typ = get_swig_target(source)
                         is_cpp = typ=='c++'
-                        if is_cpp:
-                            target_ext = '.cpp'
+                        if is_cpp: target_ext = '.cpp'
                     else:
-                        assert typ == get_swig_target(source), repr(typ)
+                        typ2 = get_swig_target(source)
+                        if typ!=typ2:
+                            log.warn('expected %r but source %r defines %r swig target' \
+                                     % (typ, source, typ2))
+                            if typ2=='c++':
+                                log.warn('resetting swig target to c++ (some targets may have .c extension)')
+                                is_cpp = True
+                                target_ext = '.cpp'
+                            else:
+                                log.warn('assuming that %r has c++ swig target' % (source))
                     target_file = os.path.join(target_dir,'%s_wrap%s' \
                                                % (name, target_ext))
                 else:
-                    log.debug('  source %s does not exist: skipping swig\'ing.' \
+                    log.warn('  source %s does not exist: skipping swig\'ing.' \
                              % (source))
                     name = ext_name[1:]
                     skip_swig = 1
                     target_file = _find_swig_target(target_dir, name)
                     if not os.path.isfile(target_file):
-                        log.debug('  target %s does not exist:\n   '\
-                                  'Assuming %s_wrap.{c,cpp} was generated with '\
-                                  '"build_src --inplace" command.' \
+                        log.warn('  target %s does not exist:\n   '\
+                                 'Assuming %s_wrap.{c,cpp} was generated with '\
+                                 '"build_src --inplace" command.' \
                                  % (target_file, name))
                         target_dir = os.path.dirname(base)
                         target_file = _find_swig_target(target_dir, name)
                         if not os.path.isfile(target_file):
-                            raise ValueError("%r missing" % (target_file,))
-                        log.debug('   Yes! Using %s as up-to-date target.' \
-                                  % (target_file))
+                            raise DistutilsSetupError("%r missing" % (target_file,))
+                        log.warn('   Yes! Using %r as up-to-date target.' \
+                                 % (target_file))
                 target_dirs.append(target_dir)
                 new_sources.append(target_file)
                 py_files.append(os.path.join(py_target_dir, name+'.py'))
@@ -578,8 +712,10 @@ class build_src(build_ext.build_ext):
         if skip_swig:
             return new_sources + py_files
 
-        map(self.mkpath, target_dirs)
-        swig = self.find_swig()
+        for d in target_dirs:
+            self.mkpath(d)
+
+        swig = self.swig or self.find_swig()
         swig_cmd = [swig, "-python"]
         if is_cpp:
             swig_cmd.append('-c++')
@@ -591,7 +727,7 @@ class build_src(build_ext.build_ext):
             if self.force or newer_group(depends, target, 'newer'):
                 log.info("%s: %s" % (os.path.basename(swig) \
                                      + (is_cpp and '++' or ''), source))
-                self.spawn(swig_cmd + self.swigflags \
+                self.spawn(swig_cmd + self.swig_opts \
                            + ["-o", target, '-outdir', py_target_dir, source])
             else:
                 log.debug("  skipping '%s' swig interface (up-to-date)" \
