@@ -6,6 +6,9 @@ import copy
 import glob
 import atexit
 import tempfile
+import subprocess
+
+from distutils.errors import DistutilsError
 
 try:
     set
@@ -22,7 +25,34 @@ __all__ = ['Configuration', 'get_numpy_include_dirs', 'default_config_dict',
            'get_script_files', 'get_lib_source_files', 'get_data_files',
            'dot_join', 'get_frame', 'minrelpath','njoin',
            'is_sequence', 'is_string', 'as_list', 'gpaths', 'get_language',
-           'quote_args', 'get_build_architecture']
+           'quote_args', 'get_build_architecture', 'get_info', 'get_pkg_info']
+
+class InstallableLib:
+    """
+    Container to hold information on an installable library.
+
+    Parameters
+    ----------
+    name : str
+        Name of the installed library.
+    build_info : dict
+        Dictionary holding build information.
+    target_dir : str
+        Absolute path specifying where to install the library.
+
+    See Also
+    --------
+    Configuration.add_installed_library
+
+    Notes
+    -----
+    The three parameters are stored as attributes with the same names.
+
+    """
+    def __init__(self, name, build_info, target_dir):
+        self.name = name
+        self.build_info = build_info
+        self.target_dir = target_dir
 
 def quote_args(args):
     # don't used _nt_quote_args as it does not check if
@@ -114,9 +144,20 @@ def njoin(*path):
 def get_mathlibs(path=None):
     """Return the MATHLIB line from numpyconfig.h
     """
-    if path is None:
-        path = os.path.join(get_numpy_include_dirs()[0], 'numpy')
-    config_file = os.path.join(path,'numpyconfig.h')
+    if path is not None:
+        config_file = os.path.join(path,'_numpyconfig.h')
+    else:
+        # Look for the file in each of the numpy include directories.
+        dirs = get_numpy_include_dirs()
+        for path in dirs:
+            fn = os.path.join(path,'_numpyconfig.h')
+            if os.path.exists(fn):
+                config_file = fn
+                break
+        else:
+            raise DistutilsError('_numpyconfig.h not found in numpy include '
+                'dirs %r' % (dirs,))
+
     fid = open(config_file)
     mathlibs = []
     s = '#define MATHLIB'
@@ -192,7 +233,7 @@ def _fix_paths(paths,local_path,include_non_existing):
             new_paths.extend(_fix_paths(n,local_path,include_non_existing))
         else:
             new_paths.append(n)
-    return map(minrelpath,new_paths)
+    return [minrelpath(p) for p in new_paths]
 
 def gpaths(paths, local_path='', include_non_existing=True):
     """Apply glob to paths and prepend local_path if needed.
@@ -315,6 +356,7 @@ def msvc_runtime_library():
         lib = {'1300' : 'msvcr70',    # MSVC 7.0
                '1310' : 'msvcr71',    # MSVC 7.1
                '1400' : 'msvcr80',    # MSVC 8
+               '1500' : 'msvcr90',    # MSVC 9 (VS 2008)
               }.get(msc_ver, None)
     else:
         lib = None
@@ -571,13 +613,58 @@ def get_frame(level=0):
             frame = frame.f_back
         return frame
 
+class SconsInfo(object):
+    """
+    Container object holding build info for building a package with scons.
+
+    Parameters
+    ----------
+    scons_path : str or None
+        Path to scons script, relative to the directory of setup.py.
+        If None, no scons script is specified. This can be useful to add only
+        pre- and post-hooks to a configuration.
+    parent_name : str or None
+        Name of the parent package (for example "numpy").
+    pre_hook : sequence of callables or None
+        Callables that are executed before scons is invoked.
+        Each callable should be defined as ``callable(*args, **kw)``.
+    post_hook : sequence of callables or None
+        Callables that are executed after scons is invoked.
+        Each callable should be defined as ``callable(*args, **kw)``.
+    source_files : list of str or None
+        List of paths to source files, relative to the directory of setup.py.
+    pkg_path : str or None
+        Path to the package for which the `SconsInfo` instance holds the
+        build info, relative to the directory of setup.py.
+
+    Notes
+    -----
+    All parameters are available as attributes of a `SconsInfo` instance.
+
+    """
+    def __init__(self, scons_path, parent_name, pre_hook,
+            post_hook, source_files, pkg_path):
+        self.scons_path = scons_path
+        self.parent_name = parent_name
+        self.pre_hook = pre_hook
+        self.post_hook = post_hook
+        self.source_files = source_files
+        if pkg_path:
+            self.pkg_path = pkg_path
+        else:
+            if scons_path:
+                self.pkg_path = os.path.dirname(scons_path)
+            else:
+                self.pkg_path = ''
+
 ######################
 
 class Configuration(object):
 
     _list_keys = ['packages', 'ext_modules', 'data_files', 'include_dirs',
-                  'libraries', 'headers', 'scripts', 'py_modules', 'scons_data']
-    _dict_keys = ['package_dir']
+                  'libraries', 'headers', 'scripts', 'py_modules', 'scons_data',
+                  'installed_libraries']
+    _dict_keys = ['package_dir', 'installed_pkg_config']
     _extra_keys = ['name', 'version']
 
     numpy_include_dirs = []
@@ -680,9 +767,15 @@ class Configuration(object):
         self.setup_name = setup_name
 
     def todict(self):
-        """Return configuration distionary suitable for passing
-        to distutils.core.setup() function.
         """
+        Return a dictionary compatible with the keyword arguments of distutils
+        setup function. 
+
+        Examples
+        --------
+        >>> setup(\**config.todict()).
+        """
+
         self._optimize_data_files()
         d = {}
         known_keys = self.list_keys + self.dict_keys + self.extra_keys
@@ -715,6 +808,7 @@ class Configuration(object):
                 raise ValueError,'Unknown option: '+key
 
     def get_distribution(self):
+        """Return the distutils distribution object for self."""
         from numpy.distutils.core import get_distribution
         return get_distribution()
 
@@ -780,7 +874,17 @@ class Configuration(object):
                        caller_level = 1):
         """Return list of subpackage configurations.
 
-        '*' in subpackage_name is handled as a wildcard.
+        Parameters
+        ----------
+        subpackage_name: str,None
+            Name of the subpackage to get the configuration. '*' in
+            subpackage_name is handled as a wildcard.
+        subpackage_path: str
+            If None, then the path is assumed to be the local path plus the
+            subpackage_name. If a setup.py file is not found in the
+            subpackage_path, then a default configuration is used.
+        parent_name: str
+            Parent name.
         """
         if subpackage_name is None:
             if subpackage_path is None:
@@ -828,8 +932,22 @@ class Configuration(object):
     def add_subpackage(self,subpackage_name,
                        subpackage_path=None,
                        standalone = False):
-        """Add subpackage to configuration.
+        """Add a sub-package to the current Configuration instance. 
+        
+        This is useful in a setup.py script for adding sub-packages to a
+        package.
+
+        Parameters
+        ----------
+        subpackage_name: str
+            name of the subpackage
+        subpackage_path: str
+            if given, the subpackage path such as the subpackage is in
+            subpackage_path / subpackage_name. If None,the subpackage is
+            assumed to be located in the local path / subpackage_name.
+        standalone: bool
         """
+
         if standalone:
             parent_name = None
         else:
@@ -856,11 +974,24 @@ class Configuration(object):
 
     def add_data_dir(self,data_path):
         """Recursively add files under data_path to data_files list.
-        Argument can be either
-        - 2-sequence (<datadir suffix>,<path to data directory>)
-        - path to data directory where python datadir suffix defaults
-          to package dir.
 
+        Recursively add files under data_path to the list of data_files to be
+        installed (and distributed). The data_path can be either a relative
+        path-name, or an absolute path-name, or a 2-tuple where the first
+        argument shows where in the install directory the data directory
+        should be installed to. 
+
+        Parameters
+        ----------
+        data_path: seq,str
+            Argument can be either
+
+                * 2-sequence (<datadir suffix>,<path to data directory>)
+                * path to data directory where python datadir suffix defaults
+                  to package dir.
+
+        Notes
+        -----
         Rules for installation paths:
           foo/bar -> (foo/bar, foo/bar) -> parent/foo/bar
           (gun, foo/bar) -> parent/gun
@@ -870,6 +1001,30 @@ class Configuration(object):
           /foo/bar -> (bar, /foo/bar) -> parent/bar
           (gun, /foo/bar) -> parent/gun
           (fun/*/gun/*, sun/foo/bar) -> parent/fun/foo/gun/bar
+
+        Examples
+        --------
+        For example suppose the source directory contains fun/foo.dat and
+        fun/bar/car.dat::
+
+            >>> self.add_data_dir('fun')
+            >>> self.add_data_dir(('sun', 'fun'))
+            >>> self.add_data_dir(('gun', '/full/path/to/fun'))
+
+        Will install data-files to the locations::
+
+            <package install directory>/
+              fun/
+                foo.dat
+                bar/
+                  car.dat
+              sun/
+                foo.dat
+                bar/
+                  car.dat
+              gun/
+                foo.dat
+                car.dat
         """
         if is_sequence(data_path):
             d, data_path = data_path
@@ -941,33 +1096,101 @@ class Configuration(object):
         for p,files in self.data_files:
             if p not in data_dict:
                 data_dict[p] = set()
-            map(data_dict[p].add,files)
+            for f in files:
+                data_dict[p].add(f)
         self.data_files[:] = [(p,list(files)) for p,files in data_dict.items()]
 
     def add_data_files(self,*files):
         """Add data files to configuration data_files.
-        Argument(s) can be either
-        - 2-sequence (<datadir prefix>,<path to data file(s)>)
-        - paths to data files where python datadir prefix defaults
-          to package dir.
+
+        Parameters
+        ----------
+        files: sequence
+            Argument(s) can be either
+
+                * 2-sequence (<datadir prefix>,<path to data file(s)>)
+                * paths to data files where python datadir prefix defaults
+                  to package dir.
+
+        Notes
+        -----
+        The form of each element of the files sequence is very flexible
+        allowing many combinations of where to get the files from the package
+        and where they should ultimately be installed on the system. The most
+        basic usage is for an element of the files argument sequence to be a
+        simple filename. This will cause that file from the local path to be
+        installed to the installation path of the self.name package (package
+        path). The file argument can also be a relative path in which case the
+        entire relative path will be installed into the package directory.
+        Finally, the file can be an absolute path name in which case the file
+        will be found at the absolute path name but installed to the package
+        path.
+
+        This basic behavior can be augmented by passing a 2-tuple in as the
+        file argument. The first element of the tuple should specify the
+        relative path (under the package install directory) where the
+        remaining sequence of files should be installed to (it has nothing to
+        do with the file-names in the source distribution). The second element
+        of the tuple is the sequence of files that should be installed. The
+        files in this sequence can be filenames, relative paths, or absolute
+        paths. For absolute paths the file will be installed in the top-level
+        package installation directory (regardless of the first argument).
+        Filenames and relative path names will be installed in the package
+        install directory under the path name given as the first element of
+        the tuple.
 
         Rules for installation paths:
-          file.txt -> (., file.txt)-> parent/file.txt
-          foo/file.txt -> (foo, foo/file.txt) -> parent/foo/file.txt
-          /foo/bar/file.txt -> (., /foo/bar/file.txt) -> parent/file.txt
-          *.txt -> parent/a.txt, parent/b.txt
-          foo/*.txt -> parent/foo/a.txt, parent/foo/b.txt
-          */*.txt -> (*, */*.txt) -> parent/c/a.txt, parent/d/b.txt
-          (sun, file.txt) -> parent/sun/file.txt
-          (sun, bar/file.txt) -> parent/sun/file.txt
-          (sun, /foo/bar/file.txt) -> parent/sun/file.txt
-          (sun, *.txt) -> parent/sun/a.txt, parent/sun/b.txt
-          (sun, bar/*.txt) -> parent/sun/a.txt, parent/sun/b.txt
-          (sun/*, */*.txt) -> parent/sun/c/a.txt, parent/d/b.txt
+
+          #. file.txt -> (., file.txt)-> parent/file.txt
+          #. foo/file.txt -> (foo, foo/file.txt) -> parent/foo/file.txt
+          #. /foo/bar/file.txt -> (., /foo/bar/file.txt) -> parent/file.txt
+          #. *.txt -> parent/a.txt, parent/b.txt
+          #. foo/*.txt -> parent/foo/a.txt, parent/foo/b.txt
+          #. */*.txt -> (*, */*.txt) -> parent/c/a.txt, parent/d/b.txt
+          #. (sun, file.txt) -> parent/sun/file.txt
+          #. (sun, bar/file.txt) -> parent/sun/file.txt
+          #. (sun, /foo/bar/file.txt) -> parent/sun/file.txt
+          #. (sun, *.txt) -> parent/sun/a.txt, parent/sun/b.txt
+          #. (sun, bar/*.txt) -> parent/sun/a.txt, parent/sun/b.txt
+          #. (sun/*, */*.txt) -> parent/sun/c/a.txt, parent/d/b.txt
+
+        An additional feature is that the path to a data-file can actually be
+        a function that takes no arguments and returns the actual path(s) to
+        the data-files. This is useful when the data files are generated while
+        building the package.
+
+        Examples
+        --------
+        Add files to the list of data_files to be included with the package.
+
+            >>> self.add_data_files('foo.dat',
+                    ('fun', ['gun.dat', 'nun/pun.dat', '/tmp/sun.dat']),
+                    'bar/cat.dat',
+                    '/full/path/to/can.dat')
+
+        will install these data files to::
+
+            <package install directory>/
+             foo.dat
+             fun/
+               gun.dat
+               nun/
+                 pun.dat
+             sun.dat
+             bar/
+               car.dat
+             can.dat
+
+        where <package install directory> is the package (or sub-package)
+        directory such as '/usr/lib/python2.4/site-packages/mypackage' ('C:
+        \\Python2.4 \\Lib \\site-packages \\mypackage') or
+        '/usr/lib/python2.4/site- packages/mypackage/mysubpackage' ('C:
+        \\Python2.4 \\Lib \\site-packages \\mypackage \\mysubpackage').
         """
 
         if len(files)>1:
-            map(self.add_data_files, files)
+            for f in files:
+                self.add_data_files(f)
             return
         assert len(files)==1
         if is_sequence(files[0]):
@@ -1032,6 +1255,10 @@ class Configuration(object):
 
     def add_include_dirs(self,*paths):
         """Add paths to configuration include directories.
+
+        Add the given sequence of paths to the beginning of the include_dirs
+        list. This list will be visible to all extension modules of the
+        current package.
         """
         include_dirs = self.paths(paths)
         dist = self.get_distribution()
@@ -1046,10 +1273,21 @@ class Configuration(object):
 
     def add_headers(self,*files):
         """Add installable headers to configuration.
-        Argument(s) can be either
-        - 2-sequence (<includedir suffix>,<path to header file(s)>)
-        - path(s) to header file(s) where python includedir suffix will default
-          to package name.
+
+        Add the given sequence of files to the beginning of the headers list.
+        By default, headers will be installed under <python-
+        include>/<self.name.replace('.','/')>/ directory. If an item of files
+        is a tuple, then its first argument specifies the actual installation
+        location relative to the <python-include> path.
+
+        Parameters
+        ----------
+        files: str, seq
+            Argument(s) can be either:
+
+                * 2-sequence (<includedir suffix>,<path to header file(s)>)
+                * path(s) to header file(s) where python includedir suffix will
+                  default to package name.
         """
         headers = []
         for path in files:
@@ -1067,6 +1305,13 @@ class Configuration(object):
 
     def paths(self,*paths,**kws):
         """Apply glob to paths and prepend local_path if needed.
+
+        Applies glob.glob(...) to each path in the sequence (if needed) and
+        pre-pends the local_path if needed. Because this is called on all
+        source lists, this allows wildcard characters to be specified in lists
+        of sources for extension modules and libraries and scripts and allows
+        path-names be relative to the source directory.
+
         """
         include_non_existing = kws.get('include_non_existing',True)
         return gpaths(paths,
@@ -1084,14 +1329,48 @@ class Configuration(object):
     def add_extension(self,name,sources,**kw):
         """Add extension to configuration.
 
-        Keywords:
-          include_dirs, define_macros, undef_macros,
-          library_dirs, libraries, runtime_library_dirs,
-          extra_objects, extra_compile_args, extra_link_args,
-          export_symbols, swig_opts, depends, language,
-          f2py_options, module_dirs
-          extra_info - dict or list of dict of keywords to be
-                       appended to keywords.
+        Create and add an Extension instance to the ext_modules list. This
+        method also takes the following optional keyword arguments that are
+        passed on to the Extension constructor.
+
+        Parameters
+        ----------
+        name: str
+            name of the extension
+        sources: seq
+            list of the sources. The list of sources may contain functions
+            (called source generators) which must take an extension instance
+            and a build directory as inputs and return a source file or list of
+            source files or None. If None is returned then no sources are
+            generated. If the Extension instance has no sources after
+            processing all source generators, then no extension module is
+            built.
+        include_dirs:
+        define_macros:
+        undef_macros:
+        library_dirs:
+        libraries:
+        runtime_library_dirs:
+        extra_objects:
+        extra_compile_args:
+        extra_link_args:
+        export_symbols:
+        swig_opts:
+        depends:
+            The depends list contains paths to files or directories that the
+            sources of the extension module depend on. If any path in the
+            depends list is newer than the extension module, then the module
+            will be rebuilt.
+        language:
+        f2py_options:
+        module_dirs:
+        extra_info: dict,list
+            dict or list of dict of keywords to be appended to keywords.
+
+        Notes
+        -----
+        The self.paths(...) method is applied to all lists that may contain
+        paths. 
         """
         ext_args = copy.copy(kw)
         ext_args['name'] = dot_join(self.name,name)
@@ -1147,36 +1426,203 @@ class Configuration(object):
         return ext
 
     def add_library(self,name,sources,**build_info):
-        """Add library to configuration.
-
-        Valid keywords for build_info:
-          depends
-          macros
-          include_dirs
-          extra_compiler_args
-          f2py_options
-          language
         """
-        build_info = copy.copy(build_info)
-        name = name #+ '__OF__' + self.name
-        build_info['sources'] = sources
+        Add library to configuration.
 
-        self._fix_paths_dict(build_info)
+        Parameters
+        ----------
+        name : str
+            Name of the extension.
+        sources : sequence
+            List of the sources. The list of sources may contain functions
+            (called source generators) which must take an extension instance
+            and a build directory as inputs and return a source file or list of
+            source files or None. If None is returned then no sources are
+            generated. If the Extension instance has no sources after
+            processing all source generators, then no extension module is
+            built.
+        build_info : dict, optional
+            The following keys are allowed:
 
-        self.libraries.append((name,build_info))
+                * depends
+                * macros
+                * include_dirs
+                * extra_compiler_args
+                * f2py_options
+                * language
+
+        """
+        self._add_library(name, sources, None, build_info)
 
         dist = self.get_distribution()
         if dist is not None:
             self.warn('distutils distribution has been initialized,'\
                       ' it may be too late to add a library '+ name)
 
+    def _add_library(self, name, sources, install_dir, build_info):
+        """Common implementation for add_library and add_installed_library. Do
+        not use directly"""
+        build_info = copy.copy(build_info)
+        name = name #+ '__OF__' + self.name
+        build_info['sources'] = sources
+
+        # Sometimes, depends is not set up to an empty list by default, and if
+        # depends is not given to add_library, distutils barfs (#1134)
+        if not build_info.has_key('depends'):
+            build_info['depends'] = []
+
+        self._fix_paths_dict(build_info)
+
+        # Add to libraries list so that it is build with build_clib
+        self.libraries.append((name, build_info))
+
+    def add_installed_library(self, name, sources, install_dir, build_info=None):
+        """
+        Similar to add_library, but the specified library is installed.
+
+        Most C libraries used with `distutils` are only used to build python
+        extensions, but libraries built through this method will be installed
+        so that they can be reused by third-party packages.
+
+        Parameters
+        ----------
+        name : str
+            Name of the installed library.
+        sources : sequence
+            List of the library's source files. See `add_library` for details.
+        install_dir : str
+            Path to install the library, relative to the current sub-package.
+        build_info : dict, optional
+            The following keys are allowed:
+
+                * depends
+                * macros
+                * include_dirs
+                * extra_compiler_args
+                * f2py_options
+                * language
+
+        Returns
+        -------
+        None
+
+        See Also
+        --------
+        add_library, add_npy_pkg_config, get_info
+
+        Notes
+        -----
+        The best way to encode the options required to link against the specified
+        C libraries is to use a "libname.ini" file, and use `get_info` to
+        retrieve the required options (see `add_npy_pkg_config` for more
+        information).
+
+        """
+        if not build_info:
+            build_info = {}
+
+        install_dir = os.path.join(self.package_path, install_dir)
+        self._add_library(name, sources, install_dir, build_info)
+        self.installed_libraries.append(InstallableLib(name, build_info, install_dir))
+
+    def add_npy_pkg_config(self, template, install_dir, subst_dict=None):
+        """
+        Generate and install a npy-pkg config file from a template.
+
+        The config file generated from `template` is installed in the
+        given install directory, using `subst_dict` for variable substitution.
+
+        Parameters
+        ----------
+        template : str
+            The path of the template, relatively to the current package path.
+        install_dir : str
+            Where to install the npy-pkg config file, relatively to the current
+            package path.
+        subst_dict : dict, optional
+            If given, any string of the form ``@key@`` will be replaced by
+            ``subst_dict[key]`` in the template file when installed. The install
+            prefix is always available through the variable ``@prefix@``, since the
+            install prefix is not easy to get reliably from setup.py.
+
+        See also
+        --------
+        add_installed_library, get_info
+
+        Notes
+        -----
+        This works for both standard installs and in-place builds, i.e. the
+        ``@prefix@`` refer to the source directory for in-place builds.
+
+        Examples
+        --------
+        ::
+
+            config.add_npy_pkg_config('foo.ini.in', 'lib', {'foo': bar})
+
+        Assuming the foo.ini.in file has the following content::
+
+            [meta]
+            Name=@foo@
+            Version=1.0
+            Description=dummy description
+
+            [default]
+            Cflags=-I@prefix@/include
+            Libs=
+
+        The generated file will have the following content::
+
+            [meta]
+            Name=bar
+            Version=1.0
+            Description=dummy description
+
+            [default]
+            Cflags=-Iprefix_dir/include
+            Libs=
+
+        and will be installed as foo.ini in the 'lib' subpath.
+
+        """
+        if subst_dict is None:
+            subst_dict = {}
+        basename = os.path.splitext(template)[0]
+        template = os.path.join(self.package_path, template)
+
+        if self.installed_pkg_config.has_key(self.name):
+            self.installed_pkg_config[self.name].append((template, install_dir,
+                subst_dict))
+        else:
+            self.installed_pkg_config[self.name] = [(template, install_dir,
+                subst_dict)]
+
+    def add_scons_installed_library(self, name, install_dir):
+        """
+        Add a scons-built installable library to distutils.
+
+        Parameters
+        ----------
+        name : str
+            The name of the library.
+        install_dir : str
+            Path to install the library, relative to the current sub-package.
+
+        """
+        install_dir = os.path.join(self.package_path, install_dir)
+        self.installed_libraries.append(InstallableLib(name, {}, install_dir))
+
     def add_sconscript(self, sconscript, subpackage_path=None,
                        standalone = False, pre_hook = None,
-                       post_hook = None, source_files = None):
+                       post_hook = None, source_files = None, package_path=None):
         """Add a sconscript to configuration.
 
         pre_hook and post hook should be sequences of callable, which will be
-        use before and after executing scons. """
+        use before and after executing scons. The callable should be defined as
+        callable(*args, **kw). It is ugly, but well, hooks are ugly anyway...
+
+        sconscript can be None, which can be useful to add only post/pre
+        hooks."""
         if standalone:
             parent_name = None
         else:
@@ -1193,34 +1639,28 @@ class Configuration(object):
         if source_files:
             full_source_files.extend([self.paths(i)[0] for i in source_files])
 
+        scons_info = SconsInfo(fullsconsname, parent_name,
+                               pre_hook, post_hook,
+                               full_source_files, package_path)
         if dist is not None:
-            dist.scons_data.append((fullsconsname,
-                                    pre_hook,
-                                    post_hook,
-                                    full_source_files,
-                                    parent_name))
+            dist.scons_data.append(scons_info)
             self.warn('distutils distribution has been initialized,'\
                       ' it may be too late to add a subpackage '+ subpackage_name)
             # XXX: we add a fake extension, to correctly initialize some
             # options in distutils command.
             dist.add_extension('', sources = [])
         else:
-            self.scons_data.append((fullsconsname,
-                                    pre_hook,
-                                    post_hook,
-                                    full_source_files,
-                                    parent_name))
+            self.scons_data.append(scons_info)
             # XXX: we add a fake extension, to correctly initialize some
             # options in distutils command.
             self.add_extension('', sources = [])
 
-    def add_configres(self):
-        from numscons import get_scons_configres_dir, get_scons_configres_filename
-        file = os.path.join(get_scons_configres_dir(), self.local_path,
-                            get_scons_configres_filename())
-
     def add_scripts(self,*files):
         """Add scripts to configuration.
+
+        Add the sequence of files to the beginning of the scripts list.
+        Scripts will be installed under the <prefix>/bin/ directory.
+
         """
         scripts = self.paths(files)
         dist = self.get_distribution()
@@ -1268,6 +1708,9 @@ class Configuration(object):
         return s
 
     def get_config_cmd(self):
+        """
+        Returns the numpy.distutils config command instance.
+        """
         cmd = get_cmd('config')
         cmd.ensure_finalized()
         cmd.dump_source = 0
@@ -1279,14 +1722,24 @@ class Configuration(object):
         return cmd
 
     def get_build_temp_dir(self):
+        """
+        Return a path to a temporary directory where temporary files should be
+        placed.
+        """
         cmd = get_cmd('build')
         cmd.ensure_finalized()
         return cmd.build_temp
 
     def have_f77c(self):
         """Check for availability of Fortran 77 compiler.
+
         Use it inside source generating function to ensure that
         setup distribution instance has been initialized.
+
+        Notes
+        -----
+        True if a Fortran 77 compiler is available (because a simple Fortran 77
+        code was able to be compiled successfully).
         """
         simple_fortran_subroutine = '''
         subroutine simple
@@ -1298,8 +1751,14 @@ class Configuration(object):
 
     def have_f90c(self):
         """Check for availability of Fortran 90 compiler.
+
         Use it inside source generating function to ensure that
         setup distribution instance has been initialized.
+
+        Notes
+        -----
+        True if a Fortran 90 compiler is available (because a simple Fortran
+        90 code was able to be compiled successfully)
         """
         simple_fortran_subroutine = '''
         subroutine simple
@@ -1329,7 +1788,10 @@ class Configuration(object):
         revision = None
         m = None
         try:
-            sin, sout = os.popen4('svnversion')
+            p = subprocess.Popen(['svnversion'], shell=True,
+                    stdout=subprocess.PIPE, stderr=STDOUT,
+                    close_fds=True)
+            sout = p.stdout
             m = re.match(r'(?P<revision>\d+)', sout.read())
         except:
             pass
@@ -1356,6 +1818,16 @@ class Configuration(object):
 
     def get_version(self, version_file=None, version_variable=None):
         """Try to get version string of a package.
+
+        Return a version string of the current package or None if the version
+        information could not be detected. 
+        
+        Notes
+        -----
+        This method scans files named
+        __version__.py, <packagename>_version.py, version.py, and
+        __svn_version__.py for string variables version, __version\__, and
+        <packagename>_version, until a version number is found.
         """
         version = getattr(self,'version',None)
         if version is not None:
@@ -1409,11 +1881,20 @@ class Configuration(object):
         return version
 
     def make_svn_version_py(self, delete=True):
-        """Generate package __svn_version__.py file from SVN revision number,
+        """Appends a data function to the data_files list that will generate
+        __svn_version__.py file to the current package directory. 
+        
+        Generate package __svn_version__.py file from SVN revision number,
         it will be removed after python exits but will be available
         when sdist, etc commands are executed.
 
+        Notes
+        -----
         If __svn_version__.py existed before, nothing is done.
+
+        This is
+        intended for working with source directories that are in an SVN
+        repository.
         """
         target = njoin(self.local_path,'__svn_version__.py')
         revision = self._get_svn_revision(self.local_path)
@@ -1445,6 +1926,10 @@ class Configuration(object):
     def make_config_py(self,name='__config__'):
         """Generate package __config__.py file containing system_info
         information used during building the package.
+
+        This file is installed to the
+        package installation directory.
+
         """
         self.py_modules.append((self.name,name,generate_config_py))
 
@@ -1456,6 +1941,9 @@ class Configuration(object):
 
     def get_info(self,*names):
         """Get resources information.
+
+        Return information (from system_info.get_info) for all of the names in
+        the argument list in a single dictionary.
         """
         from system_info import get_info, dict_append
         info_dict = {}
@@ -1485,6 +1973,121 @@ def get_numpy_include_dirs():
     # else running numpy/core/setup.py
     return include_dirs
 
+def get_npy_pkg_dir():
+    """Return the path where to find the npy-pkg-config directory."""
+    # XXX: import here for bootstrapping reasons
+    import numpy
+    d = os.path.join(os.path.dirname(numpy.__file__),
+            'core', 'lib', 'npy-pkg-config')
+    return d
+
+def get_pkg_info(pkgname, dirs=None):
+    """
+    Return library info for the given package.
+
+    Parameters
+    ----------
+    pkgname : str
+        Name of the package (should match the name of the .ini file, without
+        the extension, e.g. foo for the file foo.ini).
+    dirs : sequence, optional
+        If given, should be a sequence of additional directories where to look
+        for npy-pkg-config files. Those directories are searched prior to the
+        NumPy directory.
+
+    Returns
+    -------
+    pkginfo : class instance
+        The `LibraryInfo` instance containing the build information.
+
+    Raises
+    ------
+    PkgNotFound
+        If the package is not found.
+
+    See Also
+    --------
+    Configuration.add_npy_pkg_config, Configuration.add_installed_library,
+    get_info
+
+    """
+    from numpy.distutils.npy_pkg_config import read_config
+
+    if dirs:
+        dirs.append(get_npy_pkg_dir())
+    else:
+        dirs = [get_npy_pkg_dir()]
+    return read_config(pkgname, dirs)
+
+def get_info(pkgname, dirs=None):
+    """
+    Return an info dict for a given C library.
+
+    The info dict contains the necessary options to use the C library.
+
+    Parameters
+    ----------
+    pkgname : str
+        Name of the package (should match the name of the .ini file, without
+        the extension, e.g. foo for the file foo.ini).
+    dirs : sequence, optional
+        If given, should be a sequence of additional directories where to look
+        for npy-pkg-config files. Those directories are searched prior to the
+        NumPy directory.
+
+    Returns
+    -------
+    info : dict
+        The dictionary with build information.
+
+    Raises
+    ------
+    PkgNotFound
+        If the package is not found.
+
+    See Also
+    --------
+    Configuration.add_npy_pkg_config, Configuration.add_installed_library,
+    get_pkg_info
+
+    Examples
+    --------
+    To get the necessary information for the npymath library from NumPy:
+
+    >>> npymath_info = np.distutils.misc_util.get_info('npymath')
+    >>> npymath_info
+    {'define_macros': [], 'libraries': ['npymath'], 'library_dirs':
+    ['.../numpy/core/lib'], 'include_dirs': ['.../numpy/core/include']}
+
+    This info dict can then be used as input to a `Configuration` instance::
+
+      config.add_extension('foo', sources=['foo.c'], extra_info=npymath_info)
+
+    """
+    from numpy.distutils.npy_pkg_config import parse_flags
+    pkg_info = get_pkg_info(pkgname, dirs)
+
+    # Translate LibraryInfo instance into a build_info dict
+    info = parse_flags(pkg_info.cflags())
+    for k, v in parse_flags(pkg_info.libs()).items():
+        info[k].extend(v)
+
+    # add_extension extra_info argument is ANAL
+    info['define_macros'] = info['macros']
+    del info['macros']
+    del info['ignored']
+
+    return info
+
+def is_bootstrapping():
+    import __builtin__
+    try:
+        __builtin__.__NUMPY_SETUP__
+        return True
+    except AttributeError:
+        return False
+        __NUMPY_SETUP__ = False
+
 def scons_generate_config_py(target):
     """generate config.py file containing system_info information
     used during building the package.
@@ -1615,6 +2218,14 @@ def show():
     f.close()
     return target
 
+def msvc_version(compiler):
+    """Return version major and minor of compiler instance if it is
+    MSVC, raise an exception otherwise."""
+    if not compiler.compiler_type == "msvc":
+        raise ValueError("Compiler instance is not msvc (%s)"\
+                         % compiler.compiler_type)
+    return compiler._MSVCCompiler__version
+
 if sys.version[:3] >= '2.5':
     def get_build_architecture():
         from distutils.msvccompiler import get_build_architecture
