@@ -49,7 +49,10 @@ class $name(pu.PolyBase) :
     """
     # Limit runaway size. T_n^m has degree n*2^m
     maxpower = 16
+    # Default domain
     domain = np.array($domain)
+    # Don't let participate in array operations. Value doesn't matter.
+    __array_priority__ = 0
 
     def __init__(self, coef, domain=$domain) :
         [coef, domain] = pu.as_series([coef, domain], trim=False)
@@ -441,12 +444,12 @@ class $name(pu.PolyBase) :
         `${nick}der` : similar function for derivative.
 
         """
-        off, scl = pu.mapparms($domain, self.domain)
+        off, scl = self.mapparms()
         if lbnd is None :
             lbnd = 0
         else :
-            lbnd = off + scl*x
-        coef = ${nick}int(self.coef, m, k, lbnd, scl)
+            lbnd = off + scl*lbnd
+        coef = ${nick}int(self.coef, m, k, lbnd, 1./scl)
         return self.__class__(coef, self.domain)
 
     def deriv(self, m=1):
@@ -472,7 +475,7 @@ class $name(pu.PolyBase) :
         `${nick}int` : similar function for integration.
 
         """
-        off, scl = pu.mapparms(self.domain, $domain)
+        off, scl = self.mapparms()
         coef = ${nick}der(self.coef, m, scl)
         return self.__class__(coef, self.domain)
 
