@@ -13,6 +13,11 @@ def configuration(parent_package='',top_path=None):
     config.add_subpackage('fft')
     config.add_subpackage('linalg')
     config.add_subpackage('random')
+    config.add_subpackage('ma')
+    config.add_subpackage('matrixlib')
+    config.add_subpackage('compat')
+    config.add_subpackage('polynomial')
+    config.add_subpackage('doc')
     config.add_data_dir('doc')
     config.add_data_dir('tests')
     config.make_config_py() # installs __config__.py
